#{{{ Import modules
import numpy as np
import matplotlib.pyplot as plt
pi=np.pi
ar=np.array
#}}}

#{{{ Plot

#{{{ Plot
x=np.linspace(0,2*pi,1000)
ax=plt.plot(x, np.sin(x),label='sin(x)')[0].axes
ax.set_xlim([0, 2*pi])
#}}}

#{{{ Axis labels
ax.set_xlabel('sec', size=16)
ax.set_ylabel('$\mathsf{V/\sqrt{Hz}}$', size=16)
ax.set_title('Test Plot', size=16)
ax.figure.show()
#}}}

#{{{ Grid
ax.grid(True, color=(0.6,0.6,0.8),ls='-')
ax.grid(True, which='minor',color=(0.6,0.6,0.6),ls='--')
ax.figure.show()
#}}}

#{{{ Tick labels
for label in ax.get_xticklabels() + ax.get_yticklabels():
      label.set_fontsize(16)
#}}}

#{{{ Line properties
for line in ax.get_lines():
    line.set_lw(2)
#}}}

#{{{ Legend
ax.legend(loc='best', prop={'size':20})
ax.figure.show()
#}}}

#{{{ Axes size in figure
ax.set_position([0.16,0.1,0.78,0.8])
ax.figure.show()
#}}}

#}}}

#{{{ Save figure
ax.figure.savefig('testplot.png',format='png')
#}}}

#{{{  Kanamachi Dose Rate
import scipy.optimize as opt

#{{{ Data

#Up to 5/28
d = np.array([0.428,0.398,0.427,0.402,0.613,0.387, 0.456,0.457,0.498,0.434,0.424,0.416,0.527,0.367,0.424,0.455,0.480,0.511,0.513,0.492,0.498,0.493,0.480,0.511,0.513,0.492,0.498,0.493,0.480,0.490,0.459,0.472,0.482,0.457,0.454,0.431,0.433,0.442,0.434,0.430,0.422,0.443,0.431,0.448,0.422,0.439,0.400,0.467,0.446,0.434,0.444,0.439,0.428,0.393,0.406,0.387,0.423,0.378,0.395,0.381,0.389,0.392,0.376,0.370,0.361,0.362,0.375,0.356,0.388,0.344,0.363,0.348,0.364,0.360,0.372,0.356,0.347,0.340,0.359, 0.349,0.338,0.326,0.345,0.316,0.344,0.348,0.345,0.363,0.358,0.348, 0.347, 0.330, 0.337, 0.328, 0.353, 0.359, 0.343])


d1=np.mean(np.reshape(d[0:42],(-1,3)),1)
d2=d[42:]

d = np.hstack((d1,d2))

t = ar(range(size(d)))

ax=plt.plot(d)[0].axes
ax.grid()
ax.set_xlabel('Time since 3/25 [days]', size=16)
ax.set_ylabel('Dose Rate [$\mu$Sv/h]', size=16)
ax.figure.show()
#}}}

#{{{ Half life fit

#Half life
Th = 8.0
#Time constant
tau = Th/np.log(2)

#Fitting function
fitFunc = lambda p,t: p[0]*np.exp(-t/tau)+p[1]
errFunc = lambda p,t,y: fitFunc(p,t) -y

#Initial Guess
p0 = [0.2, 0.3]
#Least Square Fit
(p,success) = opt.leastsq(errFunc, p0, args=(t,d))

#Plot the fitting result
tfit = np.linspace(0,np.max(t),100)
dfit = fitFunc(p,tfit)
plt.plot(tfit,dfit)

#}}}

#}}}

