#{{{ Modules
import numpy as np
import matplotlib.pyplot as plt 
import scipy as sp
import scipy.signal as sg
import scipy.stats as st
import scipy.optimize as opt
import ltiutils as ltu
import psd

# Alias some frequently used objects 
ar=np.array
pi=np.pi
#}}}

#{{{ Random variable

#Gaussian Random Variable Generator
#mean = 0, sigma = 1
rv = st.norm(loc=0,scale=1)
x = np.linspace(-5,5,100)

fs = 100.0 #Sampling rate
n=1e4
v = rv.rvs(size=n)
t= np.linspace(0,n/fs,n)
plt.figure(2)
plt.plot(t,v)

#}}}

#{{{ Histogram

(n,bin,pat) = plt.hist(v, bins=50, normed=1)

#}}}

#{{{ Fitting

#Center of the bins
x=(bin[:-1]+bin[1:])/2

#Fitting function
fitFunc = lambda p,x: p[0]*np.exp(-(x-p[1])**2/p[2]**2)
errFunc = lambda p,x,y: fitFunc(p, x) - y

#Initial Guess
p0=[2500, 0, 2.5]

#Least square optimization
(p,success) = opt.leastsq(errFunc, p0, args=(x,n))

#Plot the result
xfit =np.linspace(-8,8,100) 
nfit = fitFunc(p,xfit)
plt.plot(xfit,nfit, lw=3)

#}}}

#{{{ Auto correlation

c = np.correlate(v,v,'same', True)

c = c[size(c)/2:]
c = c[:100]/np.arange(size(v),size(v)-100,-1)
tau = t[:100] - t[0]
plt.plot(tau,c)

#}}}

#{{{ PSD
(f,p)=psd.psd(v, fs, navg=10, overlap=0.5, window=sg.hanning)

#Plot
a = plt.loglog(f,p)
ax=a[0].axes
ax.set_xlabel('Frequency [Hz]', size=16)
ax.set_ylabel('Power Spectral Density [$\mathsf{V/\sqrt{Hz}}$]', size=16)
ax.grid(True, color=(0.6,0.6,0.8),ls='-')
ax.grid(True, which='minor',color=(0.6,0.6,0.6),ls='--')

#}}}

#{{{ Filter

#{{{ Analog filter

#Design a filter (2nd order butterworth low-pass filter with fc=1Hz)
#(b,a) = sg.iirfilter(N=2, Wn=2*pi*1.0, btype='lowpass', ftype='butter', analog=True)
(b,a) = sg.iirfilter(N=2, Wn=2*pi*1.0, btype='lowpass', ftype='cheby1', rp=3,  analog=True)

#Create a filter LTI object
Flt = ltu.lti(b,a)

#Bode plot
ltu.bode(Flt)

#Calculate impulse response
(ti,y) = sg.impulse(Flt)
plt.figure(2)
ax=plt.plot(ti, np.real(y))[0].axes
ax.grid()

#Nyquist plot
ltu.nyquist(Flt)
#}}}

#{{{ Digital Filter

(b,a)=sg.bilinear(Flt.num, Flt.den, fs)
plt.figure(2)
(w,h)=sg.freqz(b,a,pi*np.logspace(-3,0,1000))
plt.loglog(fs*w/(2*pi),np.abs(h))
plt.grid()

#}}}

#{{{ Apply filter

vf = sg.lfilter(b,a,v)
plt.plot(t,v,t,vf)

#}}}

#{{{ PSD
(f,p)=psd.psd(v,fs,navg=10,overlap=0.5,window=sg.hanning)
(f,pf)=psd.psd(vf,fs,navg=10,overlap=0.5,window=sg.hanning)

#Plot
a = plt.loglog(f,p,f,pf)
ax=a[0].axes
ax.set_xlabel('Frequency [Hz]', size=16)
ax.set_ylabel('Power Spectral Density [$\mathsf{V/\sqrt{Hz}}$]', size=16)

ax.grid(True, color=(0.6,0.6,0.8),ls='-')
ax.grid(True, which='minor',color=(0.6,0.6,0.6),ls='--')

#}}}

#{{{ Auto correlation

c = np.correlate(vf,vf,'same', True)

c = c[size(c)/2:]
c = c[:500]/np.arange(size(vf),size(vf)-500,-1)
tau = t[:500] - t[0]
plt.plot(tau,c)
plt.grid()
#}}}

#}}}
