'''
This is a tutorial of gtrace.
'''

#{{{ Import Modules
#First import modules

import gtrace.beam as beam
import gtrace.optcomp as opt
from gtrace.draw import drawAllBeams, drawAllOptics, transAll, rotateAll
import gtrace.sdxf as sdxf
from gtrace.units import *
import optics.gaussian as gauss

#Numpy is almost mandatory to use gtrace
import numpy as np
pi = np.pi

#}}}

#{{{ GaussianBeam object
#First, we create a beam object.

#Prepare a q-parameter
#Rw2q() converts ROC and the radius of a beam
#into a q-parameter. np.inf is infinity.
q0 = gauss.Rw2q(np.inf, 1*mm)

#Create a GaussianBeam object.
b = beam.GaussianBeam(q0=q0, wl=1064*nm)


#}}}

#{{{ Mirror object

M = opt.Mirror(HRcenter=[1,0], normAngleHR=0.0,
                 diameter=25*cm, thickness=10*cm,
                 wedgeAngle=deg2rad(3.0), inv_ROC_HR=1./2.,
                 inv_ROC_AR=0,
                 Refl_HR=0.9, Trans_HR=1-0.9,
                 Refl_AR=500*ppm, Trans_AR=1-500*ppm,
                 n=1.45, name='Mirror')

M.rotate(deg2rad(200.0))

#}}}

#{{{ Hit the mirror with the beam

beams = M.hitFromHR(b, order=2)

#}}}

#{{{ Draw

d = sdxf.Drawing()
d.layers.append(sdxf.Layer("main_beam", color=1))

drawAllBeams(d, beams.values(), drawWidth=True, sigma=6.0, drawPower=False, layer='main_beam',  fontSize=0.01)

drawAllOptics(d, [M], drawName=True)

d.saveas('/home/aso/bin/python/examples/Rinko/test.dxf')
#}}}
