#{{{ Import modules
import scipy.weave as wv
import numpy as np
import numexpr as ne
import scipy.stats as st
ar=np.array
pi=np.pi

import pyximport
pyximport.install()
import cytest
#}}}

#{{{ Simple inline call
a=100
wv.inline('printf("%d\\n",a);',['a'])
#}}}

#{{{ Hello World
a="Hello World"

code="""
std::cout << a << std::endl;
"""
wv.inline(code,['a'])
#}}}

#{{{ Search function written in Python
def pySearch(a,t):
    """
    Returns the number of occurrences of t in array a.
    """
    c=0
    for k in a:
        if k == t:
            c=c+1            
    return c
#}}}

#{{{ Better python version
def pySearch2(a,t):
    return np.sum(a==t)
#}}}

#{{{ numexpr version
def xSearch(a,t):
    return np.sum(ne.evaluate('a==t'))
#}}}

#{{{ Search function written in C

def cSearch(a,t):
    l=len(a)

    code = """
    int c=0;

    for (int i=1; i<=l; i++) {
      if (a(i) == t) {
        c++;
      }
    }
    
    return_val = c;
    """
    return wv.inline(code, ['a','t','l'], type_converters=wv.converters.blitz)
#}}}

#{{{ Compare execution times
a=np.round(50*np.sin(np.linspace(0,2*pi*100,3e6)))

#Python dumb
timeit pySearch(a,5)

#Python better
timeit pySearch2(a,5)

#numexpr
timeit xSearch(a,5)

#weave.inline
timeit cSearch(a,5)

#Cython
timeit cytest.Search(a,5)

#}}}

#{{{ Auto correlation

#{{{ Python Auto correlation
def pyAuto(x,n):
    c=np.zeros(n)
    for ii in range(n):
        c[ii]=np.sum(x[ii:]*x[:np.size(x)-ii])/np.size(x[ii:])

    return c
#}}}

#{{{ Better (?) Python Auto Correlation

def pyAuto2(x,n):
    c = np.correlate(x,x,'same', True)
    c = c[size(c)/2:]
    return  c[:n]/np.arange(size(x),size(x)-n,-1)
    
#}}}

#{{{ C Auto correlation
def cAuto(x,n):
    l=len(x)
    c=np.zeros(n)
    code = """

    for (int i=0; i<=(n-1); i++) {
        c(i)=0;
        for (int j=0; j<=(l-i-1); j++) {            
            c(i)=c(i)+x(j+i)*x(j);
        }
        c(i)=c(i)/(l-i);
    }
    """

    wv.inline(code,['x','c','n','l'],type_converters=wv.converters.blitz)


    return c
#}}}

#{{{ Load data
d=np.loadtxt('brown.dat',delimiter=',')
t=d[:,0]
x=d[:,1]
#}}}

#{{{ Run
plot(pyAuto2(x,100))
plot(cAuto(x,100))
#}}}

#{{{ Benchmark
timeit pyAuto(x,1000)
timeit cAuto(x,1000)
timeit cytest.acorr(x,1000)

#}}}

#}}}
