"""Orthogonal Frequency Division Multiplexing"""

#{{{ Import modules

import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as st
import scipy.signal as sg
pi = np.pi
ar = np.array

#}}}

#{{{ Single frequency modulated time series

#{{{ Prepare time series

fs = 2**16 #Sampling frequency
T=1 #Observation length
N = fs*T #Number of samples
t = np.linspace(0, T, N)
fr = 2**8  # (Symbol rate)
fc = 2**12   # Carrier frequency
bs = np.round((1 + np.sin(2*pi*fr/2*t))/2) #Bit stream
x1 = (bs+0.5)*np.sin(2*pi*fc*t)  #Amplitude modulated signal

#}}}

#{{{ Plot bit series
ax=plt.plot(t,bs)[0].axes
ax.set_ylim((-0.2,1.2))

#}}}

#{{{ Plot modulated carrier
ax.plot(t, x1)
ax.set_ylim((-2,2))

#}}}

#{{{ FFT
s = np.fft.fft(x1)
s = np.abs(s[:N/2])
f = np.fft.fftfreq(N, 1.0/fs)
f = f[:N/2]

ax = plt.semilogy(f, s)[0].axes
ax.set_xlim((3000, 5000))
plt.grid()

#}}}

#}}}

#{{{ Random bit stream

#{{{ Prepare time series
(b,a) = sg.iirfilter(N=4, Wn=2*pi*10.0, btype='lowpass', ftype='butter')

rv = st.norm(loc=0, scale=1) #Random variable
b = sg.lfilter(b,a,rv.rvs(size = T*fr))
bs = ( b> 0).astype(int) #Random bit stream
bs = np.repeat(bs, fs/fr) #Repeat
x2 = (bs+0.5)*np.sin(2*pi*fc*t)

#}}}

#{{{ Plot time series
plt.plot(t, bs, t, x2)
#}}}

#{{{ FFT


s = np.fft.fft(x2)
s = np.abs(s[:N/2])
f = np.fft.fftfreq(N, 1.0/fs)
f = f[:N/2]

ax=plt.semilogy(f, s)[0].axes
ax.set_xlim((3000, 5000))
plt.grid()
#}}}

#}}}

#{{{ Orthogonal carrier

fc2 = 2**12+fr   # Carrier frequency is shifted by fr
b = sg.lfilter(b,a,rv.rvs(size = T*fr))
bs = ( b> 0).astype(int) #Random bit stream
bs = np.repeat(bs, fs/fr) #Repeat
x3 = (bs+0.5)*np.sin(2*pi*fc2*t)

s = np.fft.fft(x3)
s = np.abs(s[:N/2])
f = np.fft.fftfreq(N, 1.0/fs)
f = f[:N/2]

plt.semilogy(f, s)
ax.set_xlim((3000, 5000))
#}}}

